using System;
using System.Diagnostics;
using xnaMugen.IO;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Graphics;

namespace xnaMugen.StateMachine.Controllers
{
	[StateControllerName("ModifyExplod")]
	class ModifyExplod : StateController
	{
		public ModifyExplod(StateSystem statesystem, String label, TextSection textsection)
			: base(statesystem, label, textsection)
		{
			m_animationnumber = textsection.GetAttribute<Evaluation.PrefixedExpression>("anim", null);
			m_id = textsection.GetAttribute<Evaluation.Expression>("id", null);
			m_position = textsection.GetAttribute<Evaluation.Expression>("pos", null);
			m_postype = textsection.GetAttribute<PositionType?>("postype", null);
			m_facing = textsection.GetAttribute<Evaluation.Expression>("facing", null);
			m_verticalfacing = textsection.GetAttribute<Evaluation.Expression>("vfacing", null);
			m_bindtime = textsection.GetAttribute<Evaluation.Expression>("BindTime", null);

			Evaluation.Expression exp_vel = textsection.GetAttribute<Evaluation.Expression>("vel", null);
			Evaluation.Expression exp_velocity = textsection.GetAttribute<Evaluation.Expression>("velocity", null);
			m_velocity = exp_vel ?? exp_velocity;

			m_acceleration = textsection.GetAttribute<Evaluation.Expression>("accel", null);
			m_randomdisplacement = textsection.GetAttribute<Evaluation.Expression>("random", null);
			m_removetime = textsection.GetAttribute<Evaluation.Expression>("removetime", null);
			m_supermove = textsection.GetAttribute<Evaluation.Expression>("supermove", null);
			m_supermovetime = textsection.GetAttribute<Evaluation.Expression>("supermovetime", null);
			m_pausemovetime = textsection.GetAttribute<Evaluation.Expression>("pausemovetime", null);
			m_scale = textsection.GetAttribute<Evaluation.Expression>("scale", null);
			m_spritepriority = textsection.GetAttribute<Evaluation.Expression>("sprpriority", null);
			m_drawontop = textsection.GetAttribute<Evaluation.Expression>("ontop", null);
			m_shadow = textsection.GetAttribute<Evaluation.Expression>("shadow", null);
			m_ownpalette = textsection.GetAttribute<Evaluation.Expression>("ownpal", null);
			m_removeongethit = textsection.GetAttribute<Evaluation.Expression>("removeongethit", null);
			m_explodignorehitpause = textsection.GetAttribute<Evaluation.Expression>("ignorehitpause", null);
			m_blending = textsection.GetAttribute<Blending?>("trans", null);
			m_alpha = textsection.GetAttribute<Evaluation.Expression>("alpha", null);
		}

		public override void Run(Combat.Character character)
		{
			Combat.ModifyExplodData data = CreateModifyExplodData(character);
			if (data == null) return;

			foreach (Combat.Entity entity in character.Engine.Entities)
			{
				Combat.Explod explod = character.FilterEntityAsExplod(entity, data.Id);
				if (explod == null) continue;

				explod.Modify(data);
			}
		}

		public override Boolean IsValid()
		{
			if (base.IsValid() == false) return false;

			if (Id == null) return false;

			return true;
		}

		Combat.ModifyExplodData CreateModifyExplodData(Combat.Character character)
		{
			if (character == null) throw new ArgumentNullException("character");

			Int32? animationnumber = EvaluationHelper.AsInt32(character, AnimationNumber, null);
			Int32? id = EvaluationHelper.AsInt32(character, Id, null);
			Point? location = EvaluationHelper.AsPoint(character, Position, null);
			Int32? horizfacing = EvaluationHelper.AsInt32(character, Facing, null);
			Int32? vertfacing = EvaluationHelper.AsInt32(character, VerticalFacing, null);
			Int32? bindtime = EvaluationHelper.AsInt32(character, BindTime, null);
			Vector2? velocity = EvaluationHelper.AsVector2(character, Velocity, null);
			Vector2? acceleration = EvaluationHelper.AsVector2(character, Acceleration, null);
			Point? randomdisplacement = EvaluationHelper.AsPoint(character, RandomDisplacement, null);
			Int32? removetime = EvaluationHelper.AsInt32(character, RemoveTime, null);
			Boolean? supermove = EvaluationHelper.AsBoolean(character, Supermove, null);
			Int32? supermovetime = EvaluationHelper.AsInt32(character, SupermoveTime, null);
			Int32? pausetime = EvaluationHelper.AsInt32(character, PauseMoveTime, null);
			Vector2? scale = EvaluationHelper.AsVector2(character, Scale, null);
			Int32? spritepriority = EvaluationHelper.AsInt32(character, SpritePriority, null);
			Boolean? ontop = EvaluationHelper.AsBoolean(character, DrawOnTop, null);
			Boolean? ownpalette = EvaluationHelper.AsBoolean(character, OwnPalette, null);
			Boolean? removeongethit = EvaluationHelper.AsBoolean(character, RemoveOnGetHit, null);
			Boolean? ignorehitpause = EvaluationHelper.AsBoolean(character, ExplodIgnoreHitPause, null);
			Point? alpha = EvaluationHelper.AsPoint(character, Alpha, null);

			if (id == null) return null;

			SpriteEffects? flip = null;
			if (horizfacing != null || vertfacing != null) flip = SpriteEffects.None;

			if (horizfacing == -1) flip ^= SpriteEffects.FlipHorizontally;
			if (vertfacing == -1) flip ^= SpriteEffects.FlipVertically;

			Blending? transparency = Transparency;
			if (transparency != null && transparency.Value.BlendType == BlendType.Add && transparency.Value.SourceFactor == 0 && transparency.Value.DestinationFactor == 0)
			{
				if (alpha != null)
				{
					transparency = new Blending(BlendType.Add, alpha.Value.X, alpha.Value.Y);
				}
				else
				{
					transparency = new Blending();
				}
			}

			Combat.ModifyExplodData data = new Combat.ModifyExplodData();
			data.CommonAnimation = EvaluationHelper.IsCommon(AnimationNumber, false);
			data.AnimationNumber = animationnumber;
			data.Id = id.Value;
			data.RemoveTime = removetime;
			data.Location = (Vector2?)location;
			data.PositionType = PositionType;
			data.Velocity = velocity;
			data.Acceleration = acceleration;
			data.Flip = flip;
			data.BindTime = bindtime;
			data.Random = randomdisplacement;
			data.SuperMove = supermove;
			data.SuperMoveTime = supermovetime;
			data.PauseTime = pausetime;
			data.Scale = scale;
			data.SpritePriority = spritepriority;
			data.DrawOnTop = ontop;
			data.OwnPalFx = ownpalette;
			data.RemoveOnGetHit = removeongethit;
			data.IgnoreHitPause = ignorehitpause;
			data.Transparency = transparency;

			return data;
		}

		public Evaluation.PrefixedExpression AnimationNumber
		{
			get { return m_animationnumber; }
		}

		public Evaluation.Expression Id
		{
			get { return m_id; }
		}

		public Evaluation.Expression Position
		{
			get { return m_position; }
		}

		public PositionType? PositionType
		{
			get { return m_postype; }
		}

		public Evaluation.Expression Facing
		{
			get { return m_facing; }
		}

		public Evaluation.Expression VerticalFacing
		{
			get { return m_verticalfacing; }
		}

		public Evaluation.Expression BindTime
		{
			get { return m_bindtime; }
		}

		public Evaluation.Expression Velocity
		{
			get { return m_velocity; }
		}

		public Evaluation.Expression Acceleration
		{
			get { return m_acceleration; }
		}

		public Evaluation.Expression RandomDisplacement
		{
			get { return m_randomdisplacement; }
		}

		public Evaluation.Expression RemoveTime
		{
			get { return m_removetime; }
		}

		public Evaluation.Expression Supermove
		{
			get { return m_supermove; }
		}

		public Evaluation.Expression SupermoveTime
		{
			get { return m_supermovetime; }
		}

		public Evaluation.Expression PauseMoveTime
		{
			get { return m_pausemovetime; }
		}

		public Evaluation.Expression Scale
		{
			get { return m_scale; }
		}

		public Evaluation.Expression SpritePriority
		{
			get { return m_spritepriority; }
		}

		public Evaluation.Expression DrawOnTop
		{
			get { return m_drawontop; }
		}

		public Evaluation.Expression Shadow
		{
			get { return m_shadow; }
		}

		public Evaluation.Expression OwnPalette
		{
			get { return m_ownpalette; }
		}

		public Evaluation.Expression RemoveOnGetHit
		{
			get { return m_removeongethit; }
		}

		public Evaluation.Expression ExplodIgnoreHitPause
		{
			get { return m_explodignorehitpause; }
		}

		public Blending? Transparency
		{
			get { return m_blending; }
		}

		public Evaluation.Expression Alpha
		{
			get { return m_alpha; }
		}

		#region Fields

		[DebuggerBrowsable(DebuggerBrowsableState.Never)]
		readonly Evaluation.PrefixedExpression m_animationnumber;

		[DebuggerBrowsable(DebuggerBrowsableState.Never)]
		readonly Evaluation.Expression m_id;

		[DebuggerBrowsable(DebuggerBrowsableState.Never)]
		readonly Evaluation.Expression m_position;

		[DebuggerBrowsable(DebuggerBrowsableState.Never)]
		readonly PositionType? m_postype;

		[DebuggerBrowsable(DebuggerBrowsableState.Never)]
		readonly Evaluation.Expression m_facing;

		[DebuggerBrowsable(DebuggerBrowsableState.Never)]
		readonly Evaluation.Expression m_verticalfacing;

		[DebuggerBrowsable(DebuggerBrowsableState.Never)]
		readonly Evaluation.Expression m_bindtime;

		[DebuggerBrowsable(DebuggerBrowsableState.Never)]
		readonly Evaluation.Expression m_velocity;

		[DebuggerBrowsable(DebuggerBrowsableState.Never)]
		readonly Evaluation.Expression m_acceleration;

		[DebuggerBrowsable(DebuggerBrowsableState.Never)]
		readonly Evaluation.Expression m_randomdisplacement;

		[DebuggerBrowsable(DebuggerBrowsableState.Never)]
		readonly Evaluation.Expression m_removetime;

		[DebuggerBrowsable(DebuggerBrowsableState.Never)]
		readonly Evaluation.Expression m_supermove;

		[DebuggerBrowsable(DebuggerBrowsableState.Never)]
		readonly Evaluation.Expression m_supermovetime;

		[DebuggerBrowsable(DebuggerBrowsableState.Never)]
		readonly Evaluation.Expression m_pausemovetime;

		[DebuggerBrowsable(DebuggerBrowsableState.Never)]
		readonly Evaluation.Expression m_scale;

		[DebuggerBrowsable(DebuggerBrowsableState.Never)]
		readonly Evaluation.Expression m_spritepriority;

		[DebuggerBrowsable(DebuggerBrowsableState.Never)]
		readonly Evaluation.Expression m_drawontop;

		[DebuggerBrowsable(DebuggerBrowsableState.Never)]
		readonly Evaluation.Expression m_shadow;

		[DebuggerBrowsable(DebuggerBrowsableState.Never)]
		readonly Evaluation.Expression m_ownpalette;

		[DebuggerBrowsable(DebuggerBrowsableState.Never)]
		readonly Evaluation.Expression m_removeongethit;

		[DebuggerBrowsable(DebuggerBrowsableState.Never)]
		readonly Evaluation.Expression m_explodignorehitpause;

		[DebuggerBrowsable(DebuggerBrowsableState.Never)]
		readonly Blending? m_blending;

		[DebuggerBrowsable(DebuggerBrowsableState.Never)]
		readonly Evaluation.Expression m_alpha;

		#endregion
	}
}
